@@ -6,13 +6,14 @@ import contextlib
 import os
 import shutil
 import sys
-import traceback
+import warnings
 
 import stem.util.term
 
 import darc.typing as typing
 from darc.const import DB, DB_WEB, DEBUG, FLAG_DB, PATH_ID, PATH_LN
 from darc.db import _redis_command, save_requests
+from darc.error import DatabaseOperaionFailed, render_error
 from darc.link import parse_link
 from darc.model import (HostnameModel, HostnameQueueModel, HostsModel, RequestsHistoryModel,
                         RequestsModel, RequestsQueueModel, RobotsModel, SeleniumModel,
@@ -88,22 +89,32 @@ def main():
 
     if FLAG_DB:
         while True:
-            with contextlib.suppress(Exception):
+            try:
                 with DB:
                     DB.create_tables([
                         HostnameQueueModel, RequestsQueueModel, SeleniumQueueModel,
                     ])
+            except Exception as error:
+                warning = warnings.formatwarning(error, DatabaseOperaionFailed, __file__, 94,
+                                                 'DB.create_tables([HostnameQueueModel, ...])')
+                print(render_error(warning, stem.util.term.Color.YELLOW), end='', file=sys.stderr)  # pylint: disable=no-member
+                continue
             break
 
     if SAVE_DB:
         while True:
-            with contextlib.suppress(Exception):
+            try:
                 with DB_WEB:
                     DB_WEB.create_tables([
                         HostnameModel, URLModel,
                         RobotsModel, SitemapModel, HostsModel,
                         RequestsModel, RequestsHistoryModel, SeleniumModel,
                     ])
+            except Exception as error:
+                warning = warnings.formatwarning(error, DatabaseOperaionFailed, __file__, 108,
+                                                 'DB.create_tables([HostnameModel, ...])')
+                print(render_error(warning, stem.util.term.Color.YELLOW), end='', file=sys.stderr)  # pylint: disable=no-member
+                continue
             break
 
     if DEBUG:
