@@ -4,7 +4,7 @@
 
 As websites may have authentication requirements, etc., over
 its content, the :mod:`darc.sites` module provides site
-customisation hooks to both |requests|_ and |selenium|_
+customisation hooks to both :mod:`requests` and :mod:`selenium`
 crawling processes.
 
 """
@@ -19,6 +19,16 @@ from darc.error import SiteNotFoundWarning
 from darc.link import Link
 
 SITEMAP = collections.defaultdict(lambda: 'default', {
+    # misc/special links
+    '(data)': 'data',
+    '(script)': 'script',
+    '(bitcoin)': 'bitcoin',
+    '(ed2k)': 'ed2k',
+    '(magnet)': 'magnet',
+    '(mail)': 'mail',
+    '(tel)': 'tel',
+    '(irc)': 'irc',
+
     # 'www.sample.com': 'sample',  # darc.sites.sample
 })
 
@@ -61,18 +71,18 @@ def _get_spec(link: Link) -> typing.ModuleType:
             return importlib.import_module(spec)
     except ImportError:
         warnings.warn(f'site customisation not found: {spec}', SiteNotFoundWarning)
-        return importlib.import_module(f'darc.sites.default')
+        return importlib.import_module('darc.sites.default')
 
 
 def crawler_hook(link: Link, session: typing.Session) -> typing.Response:
-    """Customisation as to |requests|_ sessions.
+    """Customisation as to :mod:`requests` sessions.
 
     Args:
         link: Link object to be crawled.
-        session (|Session|_): Session object with proxy settings.
+        session (requests.Session): Session object with proxy settings.
 
     Returns:
-        |Response|_: The final response object with crawled data.
+        requests.Response: The final response object with crawled data.
 
     See Also:
         * :data:`darc.sites.SITE_MAP`
@@ -85,14 +95,14 @@ def crawler_hook(link: Link, session: typing.Session) -> typing.Response:
 
 
 def loader_hook(link: Link, driver: typing.Driver) -> typing.Driver:
-    """Customisation as to |selenium|_ drivers.
+    """Customisation as to :mod:`selenium` drivers.
 
     Args:
         link: Link object to be loaded.
-        driver (|Chrome|_): Web driver object with proxy settings.
+        driver (selenium.webdriver.Chrome): Web driver object with proxy settings.
 
     Returns:
-        |Chrome|_: The web driver object with loaded data.
+        selenium.webdriver.Chrome: The web driver object with loaded data.
 
     See Also:
         * :data:`darc.sites.SITE_MAP`
